@@ -10,7 +10,7 @@ def do_nothing():
     pass
 
 
-def sleep(secs):
+def sleep(secs: int):
     time.sleep(secs)
 
 
@@ -23,7 +23,7 @@ def div_by_zero():
     1 / 0
 
 
-def fib(n):
+def fib(n: int):
     if n <= 1:
         return 1
     else:
