@@ -3,15 +3,22 @@ import json
 import pickle
 import warnings
 import zlib
-
+import typing as t
 import asyncio
+
 from collections.abc import Iterable
 from datetime import datetime, timedelta, timezone
 from enum import Enum
 from functools import partial
+from redis import WatchError
+from typing import Any, List, Optional
 from uuid import uuid4
 
-from redis import WatchError
+if t.TYPE_CHECKING:
+    from .results import Result
+    from .queue import Queue
+    from redis import Redis
+    from redis.client import Pipeline
 
 from rq.compat import as_text, decode_redis_hash, string_types
 from .connections import resolve_connection
@@ -39,19 +46,20 @@ class JobStatus(str, Enum):
 
 
 class Dependency:
-    def __init__(self, jobs, allow_failure: bool = False):
-        jobs = ensure_list(jobs)
+    def __init__(self, jobs: t.List[t.Union['Job', str]], allow_failure: bool = False, enqueue_at_front: bool = False):
+        dependent_jobs = ensure_list(jobs)
         if not all(
             isinstance(job, Job) or isinstance(job, str)
-            for job in jobs
+            for job in dependent_jobs
             if job
         ):
             raise ValueError("jobs: must contain objects of type Job and/or strings representing Job ids")
-        elif len(jobs) < 1:
+        elif len(dependent_jobs) < 1:
             raise ValueError("jobs: cannot be empty.")
 
-        self.dependencies = jobs
+        self.dependencies = dependent_jobs
         self.allow_failure = allow_failure
+        self.enqueue_at_front = enqueue_at_front
 
 
 # Sentinel value to mark that some of our lazily evaluated properties have not
@@ -59,14 +67,14 @@ class Dependency:
 UNEVALUATED = object()
 
 
-def cancel_job(job_id, connection=None, serializer=None, enqueue_dependents=False):
+def cancel_job(job_id: str, connection: Optional['Redis'] = None, serializer=None, enqueue_dependents: bool = False):
     """Cancels the job with the given job ID, preventing execution.  Discards
     any job info (i.e. it can't be requeued later).
     """
     Job.fetch(job_id, connection=connection, serializer=serializer).cancel(enqueue_dependents=enqueue_dependents)
 
 
-def get_current_job(connection=None, job_class=None):
+def get_current_job(connection: Optional['Redis'] = None, job_class: Optional['Job'] = None):
     """Returns the Job instance that is currently being executed.  If this
     function is invoked from outside a job context, None is returned.
     """
@@ -76,7 +84,7 @@ def get_current_job(connection=None, job_class=None):
     return _job_stack.top
 
 
-def requeue_job(job_id, connection, serializer=None):
+def requeue_job(job_id: str, connection: 'Redis', serializer=None):
     job = Job.fetch(job_id, connection=connection, serializer=serializer)
     return job.requeue()
 
@@ -88,10 +96,10 @@ class Job:
 
     # Job construction
     @classmethod
-    def create(cls, func, args=None, kwargs=None, connection=None,
-               result_ttl=None, ttl=None, status=None, description=None,
+    def create(cls, func: t.Callable[..., t.Any], args=None, kwargs=None, connection: Optional['Redis'] = None,
+               result_ttl=None, ttl=None, status: JobStatus = None, description=None,
                depends_on=None, timeout=None, id=None, origin=None, meta=None,
-               failure_ttl=None, serializer=None, *, on_success=None, on_failure=None):
+               failure_ttl=None, serializer=None, *, on_success=None, on_failure=None) -> 'Job':
         """Creates a new Job instance for the given function, arguments, and
         keyword arguments.
         """
@@ -151,6 +159,7 @@ class Job:
         # dependency could be job instance or id, or iterable thereof
         if depends_on is not None:
             if isinstance(depends_on, Dependency):
+                job.enqueue_at_front = depends_on.enqueue_at_front
                 job.allow_dependency_failures = depends_on.allow_failure
                 depends_on_list = depends_on.dependencies
             else:
@@ -169,18 +178,18 @@ class Job:
             return q.get_job_position(self._id)
         return None
 
-    def get_status(self, refresh=True):
+    def get_status(self, refresh: bool = True) -> str:
         if refresh:
             self._status = as_text(self.connection.hget(self.key, 'status'))
 
         return self._status
 
-    def set_status(self, status, pipeline=None):
+    def set_status(self, status: str, pipeline: Optional['Pipeline'] = None):
         self._status = status
-        connection = pipeline if pipeline is not None else self.connection
+        connection: 'Redis' = pipeline if pipeline is not None else self.connection
         connection.hset(self.key, 'status', self._status)
 
-    def get_meta(self, refresh=True):
+    def get_meta(self, refresh: bool = True):
         if refresh:
             meta = self.connection.hget(self.key, 'meta')
             self.meta = self.serializer.loads(meta) if meta else {}
@@ -188,35 +197,35 @@ class Job:
         return self.meta
 
     @property
-    def is_finished(self):
+    def is_finished(self) -> bool:
         return self.get_status() == JobStatus.FINISHED
 
     @property
-    def is_queued(self):
+    def is_queued(self) -> bool:
         return self.get_status() == JobStatus.QUEUED
 
     @property
-    def is_failed(self):
+    def is_failed(self) -> bool:
         return self.get_status() == JobStatus.FAILED
 
     @property
-    def is_started(self):
+    def is_started(self) -> bool:
         return self.get_status() == JobStatus.STARTED
 
     @property
-    def is_deferred(self):
+    def is_deferred(self) -> bool:
         return self.get_status() == JobStatus.DEFERRED
 
     @property
-    def is_canceled(self):
+    def is_canceled(self) -> bool:
         return self.get_status() == JobStatus.CANCELED
 
     @property
-    def is_scheduled(self):
+    def is_scheduled(self) -> bool:
         return self.get_status() == JobStatus.SCHEDULED
 
     @property
-    def is_stopped(self):
+    def is_stopped(self) -> bool:
         return self.get_status() == JobStatus.STOPPED
 
     @property
@@ -228,7 +237,7 @@ class Job:
             return self._dependency_ids[0]
 
     @property
-    def dependency(self):
+    def dependency(self) -> Optional['Job']:
         """Returns a job's first dependency. To avoid repeated Redis fetches, we cache
         job.dependency as job._dependency.
         """
@@ -241,7 +250,7 @@ class Job:
         return job
 
     @property
-    def dependent_ids(self):
+    def dependent_ids(self) -> t.List[str]:
         """Returns a list of ids of jobs whose execution depends on this
         job's successful execution."""
         return list(map(as_text, self.connection.smembers(self.dependents_key)))
@@ -356,13 +365,13 @@ class Job:
         self._data = UNEVALUATED
 
     @classmethod
-    def exists(cls, job_id, connection=None):
+    def exists(cls, job_id: str, connection: Optional['Redis'] = None) -> int:
         """Returns whether a job hash exists for the given job ID."""
         conn = resolve_connection(connection)
         return conn.exists(cls.key_for(job_id))
 
     @classmethod
-    def fetch(cls, id, connection=None, serializer=None):
+    def fetch(cls, id: str, connection: Optional['Redis'] = None, serializer=None) -> 'Job':
         """Fetches a persisted job from its corresponding Redis key and
         instantiates it.
         """
@@ -371,7 +380,7 @@ class Job:
         return job
 
     @classmethod
-    def fetch_many(cls, job_ids, connection, serializer=None):
+    def fetch_many(cls, job_ids: t.Iterable[str], connection: 'Redis', serializer=None):
         """
         Bulk version of Job.fetch
 
@@ -383,7 +392,7 @@ class Job:
                 pipeline.hgetall(cls.key_for(job_id))
             results = pipeline.execute()
 
-        jobs = []
+        jobs: t.List[Optional['Job']] = []
         for i, job_id in enumerate(job_ids):
             if results[i]:
                 job = cls(job_id, connection=connection, serializer=serializer)
@@ -394,7 +403,7 @@ class Job:
 
         return jobs
 
-    def __init__(self, id=None, connection=None, serializer=None):
+    def __init__(self, id: str = None, connection: Optional['Redis'] = None, serializer=None):
         self.connection = resolve_connection(connection)
         self._id = id
         self.created_at = utcnow()
@@ -409,26 +418,29 @@ class Job:
         self._failure_callback = UNEVALUATED
         self.description = None
         self.origin = None
-        self.enqueued_at = None
-        self.started_at = None
-        self.ended_at = None
+        self.enqueued_at: Optional[datetime] = None
+        self.started_at: Optional[datetime] = None
+        self.ended_at: Optional[datetime] = None
         self._result = None
-        self.exc_info = None
-        self.timeout = None
-        self.result_ttl = None
-        self.failure_ttl = None
-        self.ttl = None
-        self.worker_name = None
+        self._exc_info = None
+        self.timeout: Optional[float] = None
+        self.result_ttl: Optional[int] = None
+        self.failure_ttl: Optional[int] = None
+        self.ttl: Optional[int] = None
+        self.worker_name: Optional[str] = None
         self._status = None
-        self._dependency_ids = []
+        self._dependency_ids: t.List[str] = []
         self.meta = {}
         self.serializer = resolve_serializer(serializer)
         self.retries_left = None
-        # retry_intervals is a list of int e.g [60, 120, 240]
-        self.retry_intervals = None
+        self.retry_intervals: Optional[t.List[int]] = None
         self.redis_server_version = None
-        self.last_heartbeat = None
-        self.allow_dependency_failures = None
+        self.last_heartbeat: Optional[datetime] = None
+        self.allow_dependency_failures: Optional[bool] = None
+        self.enqueue_at_front: Optional[bool] = None
+
+        from .results import Result
+        self._cached_result: Optional[Result] = None
 
     def __repr__(self):  # noqa  # pragma: no cover
         return '{0}({1!r}, enqueued_at={2!r})'.format(self.__class__.__name__,
@@ -440,7 +452,6 @@ class Job:
                                        self.id,
                                        self.description)
 
-    # Job equality
     def __eq__(self, other):  # noqa
         return isinstance(other, self.__class__) and self.id == other.id
 
@@ -456,13 +467,13 @@ class Job:
             self._id = str(uuid4())
         return self._id
 
-    def set_id(self, value):
+    def set_id(self, value: str):
         """Sets a job ID for the given job."""
         if not isinstance(value, string_types):
             raise TypeError('id must be a string, not {0}'.format(type(value)))
         self._id = value
 
-    def heartbeat(self, timestamp, ttl, pipeline=None, xx=False):
+    def heartbeat(self, timestamp: datetime, ttl: int, pipeline: Optional['Pipeline'] = None, xx: bool = False):
         self.last_heartbeat = timestamp
         connection = pipeline if pipeline is not None else self.connection
         connection.hset(self.key, 'last_heartbeat', utcformat(self.last_heartbeat))
@@ -471,12 +482,12 @@ class Job:
     id = property(get_id, set_id)
 
     @classmethod
-    def key_for(cls, job_id):
+    def key_for(cls, job_id: str):
         """The Redis key that is used to store job hash under."""
         return (cls.redis_job_namespace_prefix + job_id).encode('utf-8')
 
     @classmethod
-    def dependents_key_for(cls, job_id):
+    def dependents_key_for(cls, job_id: str):
         """The Redis key that is used to store job dependents hash under."""
         return '{0}{1}:dependents'.format(cls.redis_job_namespace_prefix, job_id)
 
@@ -494,7 +505,7 @@ class Job:
     def dependencies_key(self):
         return '{0}:{1}:dependencies'.format(self.redis_job_namespace_prefix, self.id)
 
-    def fetch_dependencies(self, watch=False, pipeline=None):
+    def fetch_dependencies(self, watch: bool = False, pipeline: Optional['Pipeline'] = None):
         """
         Fetch all of a job's dependencies. If a pipeline is supplied, and
         watch is true, then set WATCH on all the keys of all dependencies.
@@ -516,7 +527,39 @@ class Job:
         return jobs
 
     @property
-    def result(self):
+    def exc_info(self) -> Optional[str]:
+        """
+        Get the latest result and returns `exc_info` only if the latest result is a failure.
+        """
+        warnings.warn("job.exc_info is deprecated, use job.latest_result() instead.",
+                      DeprecationWarning)
+
+        from .results import Result
+
+        if self.supports_redis_streams:
+            if not self._cached_result:
+                self._cached_result = self.latest_result()
+
+            if self._cached_result and self._cached_result.type == Result.Type.FAILED:
+                return self._cached_result.exc_string
+
+        return self._exc_info
+
+    def return_value(self, refresh=False) -> Any:
+        """Returns the return value of the latest execution, if it was successful"""
+        from .results import Result
+        if refresh:
+            self._cached_result = None
+
+        if self.supports_redis_streams:
+            if not self._cached_result:
+                self._cached_result = self.latest_result()
+
+            if self._cached_result and self._cached_result.type == Result.Type.SUCCESSFUL:
+                return self._cached_result.return_value
+
+    @property
+    def result(self) -> Any:
         """Returns the return value of the job.
 
         Initially, right after enqueueing a job, the return value will be
@@ -532,6 +575,20 @@ class Job:
         written back to Redis will expire after a given amount of time (500
         seconds by default).
         """
+
+        warnings.warn("job.result is deprecated, use job.return_value instead.",
+                      DeprecationWarning)
+
+        from .results import Result
+
+        if self.supports_redis_streams:
+            if not self._cached_result:
+                self._cached_result = self.latest_result()
+
+            if self._cached_result and self._cached_result.type == Result.Type.SUCCESSFUL:
+                return self._cached_result.return_value
+
+        # Fallback to old behavior of getting result from job hash
         if self._result is None:
             rv = self.connection.hget(self.key, 'result')
             if rv is not None:
@@ -539,8 +596,15 @@ class Job:
                 self._result = self.serializer.loads(rv)
         return self._result
 
-    """Backwards-compatibility accessor property `return_value`."""
-    return_value = result
+    def results(self) -> List['Result']:
+        """Returns all Result objects"""
+        from .results import Result
+        return Result.all(self, serializer=self.serializer)
+
+    def latest_result(self) -> Optional['Result']:
+        """Returns the latest Result object"""
+        from .results import Result
+        return Result.fetch_latest(self, serializer=self.serializer)
 
     def restore(self, raw_data):
         """Overwrite properties with the provided values stored in Redis"""
@@ -567,7 +631,7 @@ class Job:
         result = obj.get('result')
         if result:
             try:
-                self._result = self.serializer.loads(obj.get('result'))
+                self._result = self.serializer.loads(result)
             except Exception:
                 self._result = "Unserializable return value"
         self.timeout = parse_timeout(obj.get('timeout')) if obj.get('timeout') else None
@@ -585,7 +649,9 @@ class Job:
         dep_id = obj.get('dependency_id')  # for backwards compatibility
         self._dependency_ids = (json.loads(dep_ids.decode()) if dep_ids
                                 else [dep_id.decode()] if dep_id else [])
-        self.allow_dependency_failures = bool(int(obj.get('allow_dependency_failures'))) if obj.get('allow_dependency_failures') else None
+        allow_failures = obj.get('allow_dependency_failures')
+        self.allow_dependency_failures = bool(int(allow_failures)) if allow_failures else None
+        self.enqueue_at_front = bool(int(obj['enqueue_at_front'])) if 'enqueue_at_front' in obj else None
         self.ttl = int(obj.get('ttl')) if obj.get('ttl') else None
         self.meta = self.serializer.loads(obj.get('meta')) if obj.get('meta') else {}
 
@@ -596,10 +662,10 @@ class Job:
         raw_exc_info = obj.get('exc_info')
         if raw_exc_info:
             try:
-                self.exc_info = as_text(zlib.decompress(raw_exc_info))
+                self._exc_info = as_text(zlib.decompress(raw_exc_info))
             except zlib.error:
                 # Fallback to uncompressed string
-                self.exc_info = as_text(raw_exc_info)
+                self._exc_info = as_text(raw_exc_info)
 
     # Persistence
     def refresh(self):  # noqa
@@ -613,7 +679,7 @@ class Job:
             raise NoSuchJobError('No such job: {0}'.format(self.key))
         self.restore(data)
 
-    def to_dict(self, include_meta=True):
+    def to_dict(self, include_meta: bool = True, include_result: bool = True) -> dict:
         """
         Returns a serialization of the current job instance
 
@@ -642,13 +708,13 @@ class Job:
         if self.enqueued_at is not None:
             obj['enqueued_at'] = utcformat(self.enqueued_at)
 
-        if self._result is not None:
+        if self._result is not None and include_result:
             try:
                 obj['result'] = self.serializer.dumps(self._result)
             except:  # noqa
                 obj['result'] = "Unserializable return value"
-        if self.exc_info is not None:
-            obj['exc_info'] = zlib.compress(str(self.exc_info).encode('utf-8'))
+        if self._exc_info is not None and include_result:
+            obj['exc_info'] = zlib.compress(str(self._exc_info).encode('utf-8'))
         if self.timeout is not None:
             obj['timeout'] = self.timeout
         if self.result_ttl is not None:
@@ -669,9 +735,13 @@ class Job:
             # convert boolean to integer to avoid redis.exception.DataError
             obj["allow_dependency_failures"] = int(self.allow_dependency_failures)
 
+        if self.enqueue_at_front is not None:
+            obj["enqueue_at_front"] = int(self.enqueue_at_front)
+
         return obj
 
-    def save(self, pipeline=None, include_meta=True):
+    def save(self, pipeline: Optional['Pipeline'] = None, include_meta: bool = True,
+             include_result=True):
         """
         Dumps the current job instance to its corresponding Redis key.
 
@@ -684,13 +754,18 @@ class Job:
         key = self.key
         connection = pipeline if pipeline is not None else self.connection
 
-        mapping = self.to_dict(include_meta=include_meta)
+        mapping = self.to_dict(include_meta=include_meta, include_result=include_result)
 
         if self.get_redis_server_version() >= (4, 0, 0):
             connection.hset(key, mapping=mapping)
         else:
             connection.hmset(key, mapping)
 
+    @property
+    def supports_redis_streams(self) -> bool:
+        """Only supported by Redis server >= 5.0 is required."""
+        return self.get_redis_server_version() >= (5, 0, 0)
+
     def get_redis_server_version(self):
         """Return Redis server version of connection"""
         if not self.redis_server_version:
@@ -703,7 +778,7 @@ class Job:
         meta = self.serializer.dumps(self.meta)
         self.connection.hset(self.key, 'meta', meta)
 
-    def cancel(self, pipeline=None, enqueue_dependents=False):
+    def cancel(self, pipeline: Optional['Pipeline'] = None, enqueue_dependents: bool = False):
         """Cancels the given job, which will prevent the job from ever being
         ran (or inspected).
 
@@ -711,7 +786,7 @@ class Job:
         without worrying about the internals required to implement job
         cancellation.
 
-        You can enqueue the jobs dependents optionally, 
+        You can enqueue the jobs dependents optionally,
         Same pipelining behavior as Queue.enqueue_dependents on whether or not a pipeline is passed in.
         """
         if self.is_canceled:
@@ -734,7 +809,7 @@ class Job:
                     # Only WATCH if no pipeline passed, otherwise caller is responsible
                     if pipeline is None:
                         pipe.watch(self.dependents_key)
-                    q.enqueue_dependents(self, pipeline=pipeline)
+                    q.enqueue_dependents(self, pipeline=pipeline, exclude_job_id=self.id)
                 self._remove_from_registries(
                     pipeline=pipe,
                     remove_from_queue=True
@@ -759,16 +834,17 @@ class Job:
                     # handle it
                     raise
 
-    def requeue(self, at_front=False):
+    def requeue(self, at_front: bool = False):
         """Requeues job."""
         return self.failed_job_registry.requeue(self, at_front=at_front)
 
-    def _remove_from_registries(self, pipeline=None, remove_from_queue=True):
+    def _remove_from_registries(self, pipeline: Optional['Pipeline'] = None, remove_from_queue: bool = True):
+        from .registry import BaseRegistry
         if remove_from_queue:
             from .queue import Queue
             q = Queue(name=self.origin, connection=self.connection, serializer=self.serializer)
             q.remove(self, pipeline=pipeline)
-
+        registry: BaseRegistry
         if self.is_finished:
             from .registry import FinishedJobRegistry
             registry = FinishedJobRegistry(self.origin,
@@ -811,7 +887,7 @@ class Job:
                                            serializer=self.serializer)
             registry.remove(self, pipeline=pipeline)
 
-    def delete(self, pipeline=None, remove_from_queue=True,
+    def delete(self, pipeline: Optional['Pipeline'] = None, remove_from_queue: bool = True,
                delete_dependents=False):
         """Cancels the job and deletes the job hash from Redis. Jobs depending
         on this job can optionally be deleted as well."""
@@ -825,7 +901,7 @@ class Job:
 
         connection.delete(self.key, self.dependents_key, self.dependencies_key)
 
-    def delete_dependents(self, pipeline=None):
+    def delete_dependents(self, pipeline: Optional['Pipeline'] = None):
         """Delete jobs depending on this job."""
         connection = pipeline if pipeline is not None else self.connection
         for dependent_id in self.dependent_ids:
@@ -849,16 +925,16 @@ class Job:
             assert self is _job_stack.pop()
         return self._result
 
-    def prepare_for_execution(self, worker_name, pipeline):
+    def prepare_for_execution(self, worker_name: str, pipeline: 'Pipeline'):
         """Set job metadata before execution begins"""
         self.worker_name = worker_name
         self.last_heartbeat = utcnow()
         self.started_at = self.last_heartbeat
         self._status = JobStatus.STARTED
         mapping = {
-            'last_heartbeat': utcformat(self.last_heartbeat),
+            'last_heartbeat': utcformat(self.last_heartbeat),  # type: ignore
             'status': self._status,
-            'started_at': utcformat(self.started_at),
+            'started_at': utcformat(self.started_at),  # type: ignore
             'worker_name': worker_name
         }
         if self.get_redis_server_version() >= (4, 0, 0):
@@ -874,14 +950,14 @@ class Job:
             return coro_result
         return result
 
-    def get_ttl(self, default_ttl=None):
+    def get_ttl(self, default_ttl: Optional[int] = None):
         """Returns ttl for a job that determines how long a job will be
         persisted. In the future, this method will also be responsible
         for determining ttl for repeated jobs.
         """
         return default_ttl if self.ttl is None else self.ttl
 
-    def get_result_ttl(self, default_ttl=None):
+    def get_result_ttl(self, default_ttl: Optional[int] = None):
         """Returns ttl for a job that determines how long a jobs result will
         be persisted. In the future, this method will also be responsible
         for determining ttl for repeated jobs.
@@ -895,7 +971,8 @@ class Job:
         """
         return get_call_string(self.func_name, self.args, self.kwargs, max_length=75)
 
-    def cleanup(self, ttl=None, pipeline=None, remove_from_queue=True):
+    def cleanup(self, ttl: Optional[int] = None, pipeline: Optional['Pipeline'] = None,
+                remove_from_queue: bool = True):
         """Prepare job for eventual deletion (if needed). This method is usually
         called after successful execution. How long we persist the job and its
         result depends on the value of ttl:
@@ -939,7 +1016,7 @@ class Job:
         index = max(number_of_intervals - self.retries_left, 0)
         return self.retry_intervals[index]
 
-    def retry(self, queue, pipeline):
+    def retry(self, queue: 'Queue', pipeline: 'Pipeline'):
         """Requeue or schedule this job for execution"""
         retry_interval = self.get_retry_interval()
         self.retries_left = self.retries_left - 1
@@ -950,7 +1027,7 @@ class Job:
         else:
             queue.enqueue_job(self, pipeline=pipeline)
 
-    def register_dependency(self, pipeline=None):
+    def register_dependency(self, pipeline: Optional['Pipeline'] = None):
         """Jobs may have dependencies. Jobs are enqueued only if the jobs they
         depend on are successfully performed. We record this relation as
         a reverse dependency (a Redis set), with a key that looks something
@@ -982,7 +1059,8 @@ class Job:
         return [Job.key_for(_id.decode())
                 for _id in dependencies]
 
-    def dependencies_are_met(self, parent_job=None, pipeline=None):
+    def dependencies_are_met(self, parent_job: Optional['Job'] = None,
+                             pipeline: Optional['Pipeline'] = None, exclude_job_id: str = None):
         """Returns a boolean indicating if all of this job's dependencies are _FINISHED_
 
         If a pipeline is passed, all dependencies are WATCHed.
@@ -1001,13 +1079,18 @@ class Job:
         dependencies_ids = {_id.decode()
                             for _id in connection.smembers(self.dependencies_key)}
 
+        if exclude_job_id:
+            dependencies_ids.discard(exclude_job_id)
+            if parent_job and parent_job.id == exclude_job_id:
+                parent_job = None
+
         if parent_job:
-            # If parent job is canceled, no need to check for status
+            # If parent job is canceled, treat dependency as failed
             # If parent job is not finished, we should only continue
             # if this job allows parent job to fail
             dependencies_ids.discard(parent_job.id)
             if parent_job._status == JobStatus.CANCELED:
-                pass
+                return False
             elif parent_job._status == JobStatus.FAILED and not self.allow_dependency_failures:
                 return False
 
@@ -1038,7 +1121,7 @@ _job_stack = LocalStack()
 
 
 class Retry:
-    def __init__(self, max, interval=0):
+    def __init__(self, max, interval: int = 0):
         """`interval` can be a positive number or a list of ints"""
         super().__init__()
         if max < 1:
