@@ -1,17 +1,21 @@
 import uuid
 import sys
 import warnings
-
+import typing as t
 from collections import namedtuple
 from datetime import datetime, timezone
 
 from redis import WatchError
 
+if t.TYPE_CHECKING:
+    from redis import Redis
+    from redis.client import Pipeline
+
 from .compat import as_text, string_types, total_ordering
 from .connections import resolve_connection
 from .defaults import DEFAULT_RESULT_TTL
 from .exceptions import DequeueTimeout, NoSuchJobError
-from .job import Job, JobStatus, Dependency
+from .job import Job, JobStatus
 from .serializers import resolve_serializer
 from .utils import backend_class, get_version, import_attribute, parse_timeout, utcnow
 
@@ -23,7 +27,7 @@ def compact(lst):
 class EnqueueData(namedtuple('EnqueueData', ["func", "args", "kwargs", "timeout",
                                              "result_ttl", "ttl", "failure_ttl",
                                              "description", "job_id",
-                                             "at_front", "meta", "retry"])):
+                                             "at_front", "meta", "retry", "on_success", "on_failure"])):
     """Helper type to use when calling enqueue_many
     NOTE: Does not support `depends_on` yet.
     """
@@ -33,13 +37,13 @@ class EnqueueData(namedtuple('EnqueueData', ["func", "args", "kwargs", "timeout"
 
 @total_ordering
 class Queue:
-    job_class = Job
-    DEFAULT_TIMEOUT = 180  # Default timeout seconds.
-    redis_queue_namespace_prefix = 'rq:queue:'
-    redis_queues_keys = 'rq:queues'
+    job_class: t.Type['Job'] = Job
+    DEFAULT_TIMEOUT: int = 180  # Default timeout seconds.
+    redis_queue_namespace_prefix: str = 'rq:queue:'
+    redis_queues_keys: str = 'rq:queues'
 
     @classmethod
-    def all(cls, connection=None, job_class=None, serializer=None):
+    def all(cls, connection: t.Optional['Redis'] = None, job_class: t.Optional[t.Type['Job']] = None, serializer=None):
         """Returns an iterable of all Queues.
         """
         connection = resolve_connection(connection)
@@ -54,7 +58,8 @@ class Queue:
                 if rq_key]
 
     @classmethod
-    def from_queue_key(cls, queue_key, connection=None, job_class=None, serializer=None):
+    def from_queue_key(cls, queue_key, connection: t.Optional['Redis'] = None,
+                       job_class: t.Optional[t.Type['Job']] = None, serializer=None):
         """Returns a Queue instance, based on the naming conventions for naming
         the internal Redis keys.  Can be used to reverse-lookup Queues by their
         Redis keys.
@@ -65,7 +70,7 @@ class Queue:
         name = queue_key[len(prefix):]
         return cls(name, connection=connection, job_class=job_class, serializer=serializer)
 
-    def __init__(self, name='default', default_timeout=None, connection=None,
+    def __init__(self, name='default', default_timeout=None, connection: t.Optional['Redis'] = None,
                  is_async=True, job_class=None, serializer=None, **kwargs):
         self.connection = resolve_connection(connection)
         prefix = self.redis_queue_namespace_prefix
@@ -143,7 +148,7 @@ class Queue:
         script = self.connection.register_script(script)
         return script(keys=[self.key])
 
-    def delete(self, delete_jobs=True):
+    def delete(self, delete_jobs: bool = True):
         """Deletes the queue. If delete_jobs is true it removes all the associated messages on the queue first."""
         if delete_jobs:
             self.empty()
@@ -162,7 +167,7 @@ class Queue:
         """Returns whether the current queue is async."""
         return bool(self._is_async)
 
-    def fetch_job(self, job_id):
+    def fetch_job(self, job_id: str):
         try:
             job = self.job_class.fetch(job_id, connection=self.connection, serializer=self.serializer)
         except NoSuchJobError:
@@ -171,7 +176,7 @@ class Queue:
             if job.origin == self.name:
                 return job
 
-    def get_job_position(self, job_or_id):
+    def get_job_position(self, job_or_id: t.Union[Job, str]):
         """Returns the position of a job within the queue
 
         Using Redis before 6.0.6 and redis-py before 3.5.4 has a complexity of
@@ -192,7 +197,7 @@ class Queue:
             return self.job_ids.index(job_id)
         return None
 
-    def get_job_ids(self, offset=0, length=-1):
+    def get_job_ids(self, offset: int = 0, length: int = -1):
         """Returns a slice of job IDs in the queue."""
         start = offset
         if length >= 0:
@@ -202,23 +207,23 @@ class Queue:
         return [as_text(job_id) for job_id in
                 self.connection.lrange(self.key, start, end)]
 
-    def get_jobs(self, offset=0, length=-1):
+    def get_jobs(self, offset: int = 0, length: int = -1):
         """Returns a slice of jobs in the queue."""
         job_ids = self.get_job_ids(offset, length)
         return compact([self.fetch_job(job_id) for job_id in job_ids])
 
     @property
-    def job_ids(self):
+    def job_ids(self) -> t.List[str]:
         """Returns a list of all job IDS in the queue."""
         return self.get_job_ids()
 
     @property
-    def jobs(self):
+    def jobs(self) -> t.List['Job']:
         """Returns a list of all (valid) jobs in the queue."""
         return self.get_jobs()
 
     @property
-    def count(self):
+    def count(self) -> int:
         """Returns a count of all messages in the queue."""
         return self.connection.llen(self.key)
 
@@ -259,7 +264,7 @@ class Queue:
         from rq.registry import CanceledJobRegistry
         return CanceledJobRegistry(queue=self, job_class=self.job_class, serializer=self.serializer)
 
-    def remove(self, job_or_id, pipeline=None):
+    def remove(self, job_or_id: t.Union['Job', str], pipeline: t.Optional['Pipeline'] = None):
         """Removes Job from queue, accepts either a Job instance or ID."""
         job_id = job_or_id.id if isinstance(job_or_id, self.job_class) else job_or_id
 
@@ -283,7 +288,7 @@ class Queue:
             if self.job_class.exists(job_id, self.connection):
                 self.connection.rpush(self.key, job_id)
 
-    def push_job_id(self, job_id, pipeline=None, at_front=False):
+    def push_job_id(self, job_id: str, pipeline: t.Optional['Pipeline'] = None, at_front=False):
         """Pushes a job ID on the corresponding Redis queue.
         'at_front' allows you to push the job onto the front instead of the back of the queue"""
         connection = pipeline if pipeline is not None else self.connection
@@ -292,11 +297,11 @@ class Queue:
         else:
             connection.rpush(self.key, job_id)
 
-    def create_job(self, func, args=None, kwargs=None, timeout=None,
+    def create_job(self, func: t.Callable[..., t.Any], args=None, kwargs=None, timeout=None,
                    result_ttl=None, ttl=None, failure_ttl=None,
                    description=None, depends_on=None, job_id=None,
                    meta=None, status=JobStatus.QUEUED, retry=None, *,
-                   on_success=None, on_failure=None):
+                   on_success=None, on_failure=None) -> Job:
         """Creates a job based on parameters given."""
         timeout = parse_timeout(timeout)
 
@@ -325,16 +330,9 @@ class Queue:
             job.retries_left = retry.max
             job.retry_intervals = retry.intervals
 
-        if isinstance(depends_on, Dependency):
-            job.allow_dependency_failures = depends_on.allow_failure
-
         return job
 
-    def setup_dependencies(
-        self,
-        job,
-        pipeline=None
-    ):
+    def setup_dependencies(self, job: 'Job', pipeline: t.Optional['Pipeline'] = None):
         # If a _dependent_ job depends on any unfinished job, register all the
         # _dependent_ job's dependencies instead of enqueueing it.
         #
@@ -386,10 +384,10 @@ class Queue:
             pipeline.multi()  # Ensure pipeline in multi mode before returning to caller
         return job
 
-    def enqueue_call(self, func, args=None, kwargs=None, timeout=None,
+    def enqueue_call(self, func: t.Callable[..., t.Any], args=None, kwargs=None, timeout=None,
                      result_ttl=None, ttl=None, failure_ttl=None, description=None,
-                     depends_on=None, job_id=None, at_front=False, meta=None,
-                     retry=None, on_success=None, on_failure=None, pipeline=None):
+                     depends_on=None, job_id: str = None, at_front: bool = False, meta=None,
+                     retry=None, on_success=None, on_failure=None, pipeline=None) -> Job:
         """Creates a job to represent the delayed function call and enqueues it.
 
         It is much like `.enqueue()`, except that it takes the function's args
@@ -417,21 +415,17 @@ class Queue:
     def prepare_data(func, args=None, kwargs=None, timeout=None,
                      result_ttl=None, ttl=None, failure_ttl=None,
                      description=None, job_id=None,
-                     at_front=False, meta=None, retry=None):
+                     at_front=False, meta=None, retry=None, on_success=None, on_failure=None) -> EnqueueData:
         # Need this till support dropped for python_version < 3.7, where defaults can be specified for named tuples
         # And can keep this logic within EnqueueData
         return EnqueueData(
             func, args, kwargs, timeout,
             result_ttl, ttl, failure_ttl,
             description, job_id,
-            at_front, meta, retry
+            at_front, meta, retry, on_success, on_failure
         )
 
-    def enqueue_many(
-        self,
-        job_datas,
-        pipeline=None
-    ):
+    def enqueue_many(self, job_datas, pipeline: t.Optional['Pipeline'] = None) -> t.List[Job]:
         """
         Creates multiple jobs (created via `Queue.prepare_data` calls)
         to represent the delayed function calls and enqueues them.
@@ -446,7 +440,9 @@ class Queue:
                     depends_on=None,
                     job_id=job_data.job_id, meta=job_data.meta, status=JobStatus.QUEUED,
                     timeout=job_data.timeout,
-                    retry=job_data.retry
+                    retry=job_data.retry,
+                    on_success=job_data.on_success,
+                    on_failure=job_data.on_failure
                 ),
                 pipeline=pipe,
                 at_front=job_data.at_front
@@ -457,7 +453,7 @@ class Queue:
             pipe.execute()
         return jobs
 
-    def run_job(self, job):
+    def run_job(self, job: 'Job') -> Job:
         job.perform()
         job.set_status(JobStatus.FINISHED)
         job.save(include_meta=False)
@@ -465,7 +461,7 @@ class Queue:
         return job
 
     @classmethod
-    def parse_args(cls, f, *args, **kwargs):
+    def parse_args(cls, f: t.Union[t.Callable[..., t.Any], str], *args, **kwargs):
         """
         Parses arguments passed to `queue.enqueue()` and `queue.enqueue_at()`
 
@@ -520,7 +516,7 @@ class Queue:
             pipeline=pipeline
         )
 
-    def enqueue_at(self, datetime, f, *args, **kwargs):
+    def enqueue_at(self, datetime: datetime, f, *args, **kwargs):
         """Schedules a job to be enqueued at specified time"""
 
         (f, timeout, description, result_ttl, ttl, failure_ttl,
@@ -531,10 +527,11 @@ class Queue:
                               failure_ttl=failure_ttl, description=description,
                               depends_on=depends_on, job_id=job_id, meta=meta, retry=retry,
                               on_success=on_success, on_failure=on_failure)
-
+        if at_front:
+            job.enqueue_at_front = True
         return self.schedule_job(job, datetime, pipeline=pipeline)
 
-    def schedule_job(self, job, datetime, pipeline=None):
+    def schedule_job(self, job: 'Job', datetime: datetime, pipeline: t.Optional['Pipeline'] = None):
         """Puts job on ScheduledJobRegistry"""
         from .registry import ScheduledJobRegistry
         registry = ScheduledJobRegistry(queue=self)
@@ -554,7 +551,7 @@ class Queue:
         return self.enqueue_at(datetime.now(timezone.utc) + time_delta,
                                func, *args, **kwargs)
 
-    def enqueue_job(self, job, pipeline=None, at_front=False):
+    def enqueue_job(self, job: 'Job', pipeline: t.Optional['Pipeline'] = None, at_front: bool = False) -> Job:
         """Enqueues a job for delayed execution.
 
         If Queue is instantiated with is_async=False, job is executed immediately.
@@ -563,6 +560,7 @@ class Queue:
 
         # Add Queue key set
         pipe.sadd(self.redis_queues_keys, self.key)
+        job.redis_server_version = self.get_redis_server_version()
         job.set_status(JobStatus.QUEUED, pipeline=pipe)
 
         job.origin = self.name
@@ -584,7 +582,7 @@ class Queue:
 
         return job
 
-    def run_sync(self, job):
+    def run_sync(self, job: 'Job') -> 'Job':
         with self.connection.pipeline() as pipeline:
             job.prepare_for_execution('sync', pipeline)
 
@@ -600,7 +598,7 @@ class Queue:
 
         return job
 
-    def enqueue_dependents(self, job, pipeline=None):
+    def enqueue_dependents(self, job: 'Job', pipeline: t.Optional['Pipeline'] = None, exclude_job_id=None):
         """Enqueues all jobs in the given job's dependents set and clears it.
 
         When called without a pipeline, this method uses WATCH/MULTI/EXEC.
@@ -636,6 +634,7 @@ class Queue:
                     ) if dependent_job and dependent_job.dependencies_are_met(
                         parent_job=job,
                         pipeline=pipe,
+                        exclude_job_id=exclude_job_id,
                     )
                 ]
 
@@ -645,16 +644,19 @@ class Queue:
                     break
 
                 for dependent in jobs_to_enqueue:
+                    enqueue_at_front = dependent.enqueue_at_front or False
+
                     registry = DeferredJobRegistry(dependent.origin,
                                                    self.connection,
                                                    job_class=self.job_class,
                                                    serializer=self.serializer)
                     registry.remove(dependent, pipeline=pipe)
+
                     if dependent.origin == self.name:
-                        self.enqueue_job(dependent, pipeline=pipe)
+                        self.enqueue_job(dependent, pipeline=pipe, at_front=enqueue_at_front)
                     else:
                         queue = self.__class__(name=dependent.origin, connection=self.connection)
-                        queue.enqueue_job(dependent, pipeline=pipe)
+                        queue.enqueue_job(dependent, pipeline=pipe, at_front=enqueue_at_front)
 
                 # Only delete dependents_key if all dependents have been enqueued
                 if len(jobs_to_enqueue) == len(dependent_job_ids):
@@ -680,7 +682,7 @@ class Queue:
         return as_text(self.connection.lpop(self.key))
 
     @classmethod
-    def lpop(cls, queue_keys, timeout, connection=None):
+    def lpop(cls, queue_keys, timeout, connection: t.Optional['Redis'] = None):
         """Helper method.  Intermediate method to abstract away from some
         Redis API details, where LPOP accepts only a single key, whereas BLPOP
         accepts multiple.  So if we want the non-blocking LPOP, we need to
@@ -710,7 +712,8 @@ class Queue:
             return None
 
     @classmethod
-    def dequeue_any(cls, queues, timeout, connection=None, job_class=None, serializer=None):
+    def dequeue_any(cls, queues, timeout, connection: t.Optional['Redis'] = None,
+                    job_class: t.Optional[t.Type['Job']] = None, serializer=None):
         """Class method returning the job_class instance at the front of the given
         set of Queues, where the order of the queues is important.
 
