@@ -104,7 +104,7 @@ class RQScheduler:
         """Returns names of queue it successfully acquires lock on"""
         successful_locks = set()
         pid = os.getpid()
-        self.log.info("Trying to acquire locks for %s", ", ".join(self._queue_names))
+        self.log.debug("Trying to acquire locks for %s", ", ".join(self._queue_names))
         for name in self._queue_names:
             if self.connection.set(self.get_locking_key(name), pid, nx=True, ex=self.interval + 60):
                 successful_locks.add(name)
@@ -122,7 +122,7 @@ class RQScheduler:
 
         return successful_locks
 
-    def prepare_registries(self, queue_names=None):
+    def prepare_registries(self, queue_names: str = None):
         """Prepare scheduled job registries for use"""
         self._scheduled_job_registries = []
         if not queue_names:
@@ -133,7 +133,7 @@ class RQScheduler:
             )
 
     @classmethod
-    def get_locking_key(cls, name):
+    def get_locking_key(cls, name: str):
         """Returns scheduler key for a given queue name"""
         return SCHEDULER_LOCKING_KEY_TEMPLATE % name
 
@@ -162,7 +162,7 @@ class RQScheduler:
                 )
                 for job in jobs:
                     if job is not None:
-                        queue.enqueue_job(job, pipeline=pipeline)
+                        queue.enqueue_job(job, pipeline=pipeline, at_front=bool(job.enqueue_at_front))
                         registry.remove(job, pipeline=pipeline)
                 pipeline.execute()
         self._status = self.Status.STARTED
@@ -184,7 +184,7 @@ class RQScheduler:
                        ", ".join(self.acquired_locks))
         if len(self._queue_names) > 1:
             with self.connection.pipeline() as pipeline:
-                for name in self._queue_names:
+                for name in self._acquired_locks:
                     key = self.get_locking_key(name)
                     pipeline.expire(key, self.interval + 60)
                 pipeline.execute()
