@@ -8,11 +8,17 @@ import sys
 import time
 import traceback
 import warnings
+import typing as t
+
+if t.TYPE_CHECKING:
+    from redis import Redis
+    from redis.client import Pipeline
 
 from datetime import timedelta
 from enum import Enum
 from uuid import uuid4
 from random import shuffle
+from typing import Callable, List, Optional
 
 try:
     from signal import SIGKILL
@@ -34,10 +40,10 @@ from .job import Job, JobStatus
 from .logutils import setup_loghandlers
 from .queue import Queue
 from .registry import FailedJobRegistry, StartedJobRegistry, clean_registries
+from .results import Result
 from .scheduler import RQScheduler
 from .suspension import is_suspended
-from .timeouts import (JobTimeoutException, HorseMonitorTimeoutException,
-                       UnixSignalDeathPenalty, TimerDeathPenalty)
+from .timeouts import JobTimeoutException, HorseMonitorTimeoutException, UnixSignalDeathPenalty
 from .utils import (backend_class, ensure_list, get_version,
                     make_colorizer, utcformat, utcnow, utcparse)
 from .version import VERSION
@@ -61,8 +67,8 @@ class StopRequested(Exception):
     pass
 
 
-def compact(l):
-    return [x for x in l if x is not None]
+def compact(a_list):
+    return [x for x in a_list if x is not None]
 
 
 _signames = dict((getattr(signal, signame), signame)
@@ -107,7 +113,14 @@ class Worker:
     max_connection_wait_time = 60.0
 
     @classmethod
-    def all(cls, connection=None, job_class=None, queue_class=None, queue=None, serializer=None):
+    def all(
+        cls,
+        connection: t.Optional['Redis'] = None,
+        job_class: t.Type['Job'] = None,
+        queue_class: t.Optional[t.Type['Queue']] = None,
+        queue: t.Optional['Queue'] = None,
+        serializer=None
+    ) -> t.List['Worker']:
         """Returns an iterable of all Workers.
         """
         if queue:
@@ -124,18 +137,18 @@ class Worker:
         return compact(workers)
 
     @classmethod
-    def all_keys(cls, connection=None, queue=None):
+    def all_keys(cls, connection: t.Optional['Redis'] = None, queue: t.Optional['Queue'] = None):
         return [as_text(key)
                 for key in get_keys(queue=queue, connection=connection)]
 
     @classmethod
-    def count(cls, connection=None, queue=None):
+    def count(cls, connection: t.Optional['Redis'] = None, queue: t.Optional['Queue'] = None):
         """Returns the number of workers by queue or connection"""
         return len(get_keys(queue=queue, connection=connection))
 
     @classmethod
-    def find_by_key(cls, worker_key, connection=None, job_class=None,
-                    queue_class=None, serializer=None):
+    def find_by_key(cls, worker_key: str, connection: t.Optional['Redis'] = None, job_class: t.Type['Job'] = None,
+                    queue_class: t.Type['Queue'] = None, serializer=None):
         """Returns a Worker instance, based on the naming conventions for
         naming the internal Redis keys.  Can be used to reverse-lookup Workers
         by their Redis keys.
@@ -158,13 +171,13 @@ class Worker:
 
         return worker
 
-    def __init__(self, queues, name=None, default_result_ttl=DEFAULT_RESULT_TTL,
-                 connection=None, exc_handler=None, exception_handlers=None,
-                 default_worker_ttl=DEFAULT_WORKER_TTL, job_class=None,
-                 queue_class=None, log_job_description=True,
+    def __init__(self, queues, name: t.Optional[str] = None, default_result_ttl=DEFAULT_RESULT_TTL,
+                 connection: t.Optional['Redis'] = None, exc_handler=None, exception_handlers=None,
+                 default_worker_ttl=DEFAULT_WORKER_TTL, job_class: t.Type['Job'] = None,
+                 queue_class=None, log_job_description: bool = True,
                  job_monitoring_interval=DEFAULT_JOB_MONITORING_INTERVAL,
-                 disable_default_exception_handler=False,
-                 prepare_for_work=True, serializer=None):  # noqa
+                 disable_default_exception_handler: bool = False,
+                 prepare_for_work: bool = True, serializer=None):  # noqa
         if connection is None:
             connection = get_current_connection()
         self.connection = connection
@@ -183,57 +196,65 @@ class Worker:
                   if isinstance(q, string_types) else q
                   for q in ensure_list(queues)]
 
-        self.name = name or uuid4().hex
+        self.name: str = name or uuid4().hex
         self.queues = queues
         self.validate_queues()
         self._ordered_queues = self.queues[:]
-        self._exc_handlers = []
+        self._exc_handlers: List[Callable] = []
 
         self.default_result_ttl = default_result_ttl
         self.default_worker_ttl = default_worker_ttl
         self.job_monitoring_interval = job_monitoring_interval
 
-        self._state = 'starting'
-        self._is_horse = False
-        self._horse_pid = 0
-        self._stop_requested = False
+        self._state: str = 'starting'
+        self._is_horse: bool = False
+        self._horse_pid: int = 0
+        self._stop_requested: bool = False
         self._stopped_job_id = None
 
         self.log = logger
         self.log_job_description = log_job_description
         self.last_cleaned_at = None
-        self.successful_job_count = 0
-        self.failed_job_count = 0
-        self.total_working_time = 0
-        self.current_job_working_time = 0
+        self.successful_job_count: int = 0
+        self.failed_job_count: int = 0
+        self.total_working_time: int = 0
+        self.current_job_working_time: int = 0
         self.birth_date = None
-        self.scheduler = None
+        self.scheduler: t.Optional[RQScheduler] = None
         self.pubsub = None
         self.pubsub_thread = None
 
         self.disable_default_exception_handler = disable_default_exception_handler
 
         if prepare_for_work:
-            self.hostname = socket.gethostname()
-            self.pid = os.getpid()
+            self.hostname: Optional[str] = socket.gethostname()
+            self.pid: Optional[int] = os.getpid()
             try:
                 connection.client_setname(self.name)
             except redis.exceptions.ResponseError:
                 warnings.warn(
-                    'CLIENT command not supported, setting ip_address to unknown',
+                    'CLIENT SETNAME command not supported, setting ip_address to unknown',
                     Warning
                 )
                 self.ip_address = 'unknown'
             else:
-                self.ip_address = [
+                client_adresses = [
                     client['addr']
                     for client in connection.client_list()
                     if client['name'] == self.name
-                ][0]
+                ]
+                if len(client_adresses) > 0:
+                    self.ip_address = client_adresses[0]
+                else:
+                    warnings.warn(
+                        'CLIENT LIST command not supported, setting ip_address to unknown',
+                        Warning
+                    )
+                    self.ip_address = 'unknown'
         else:
             self.hostname = None
             self.pid = None
-            self.ip_address = None
+            self.ip_address = 'unknown'
 
         if isinstance(exception_handlers, (list, tuple)):
             for handler in exception_handlers:
@@ -271,6 +292,11 @@ class Worker:
         """Returns the worker's Redis hash key."""
         return PUBSUB_CHANNEL_TEMPLATE % self.name
 
+    @property
+    def supports_redis_streams(self) -> bool:
+        """Only supported by Redis server >= 5.0 is required."""
+        return self.get_redis_server_version() >= (5, 0, 0)
+
     @property
     def horse_pid(self):
         """The horse's process ID.  Only available in the worker.  Will return
@@ -361,7 +387,7 @@ class Worker:
         if death_timestamp is not None:
             return utcparse(as_text(death_timestamp))
 
-    def set_state(self, state, pipeline=None):
+    def set_state(self, state, pipeline: t.Optional['Pipeline'] = None):
         self._state = state
         connection = pipeline if pipeline is not None else self.connection
         connection.hset(self.key, 'state', state)
@@ -387,12 +413,12 @@ class Worker:
 
     state = property(_get_state, _set_state)
 
-    def set_current_job_working_time(self, current_job_working_time, pipeline=None):
+    def set_current_job_working_time(self, current_job_working_time, pipeline: t.Optional['Pipeline'] = None):
         self.current_job_working_time = current_job_working_time
         connection = pipeline if pipeline is not None else self.connection
         connection.hset(self.key, 'current_job_working_time', current_job_working_time)
 
-    def set_current_job_id(self, job_id, pipeline=None):
+    def set_current_job_id(self, job_id: t.Optional[str] = None, pipeline: t.Optional['Pipeline'] = None):
         connection = pipeline if pipeline is not None else self.connection
 
         if job_id is None:
@@ -400,7 +426,7 @@ class Worker:
         else:
             connection.hset(self.key, 'current_job', job_id)
 
-    def get_current_job_id(self, pipeline=None):
+    def get_current_job_id(self, pipeline: t.Optional['Pipeline'] = None):
         connection = pipeline if pipeline is not None else self.connection
         return as_text(connection.hget(self.key, 'current_job'))
 
@@ -546,8 +572,8 @@ class Worker:
     def reorder_queues(self, reference_queue):
         pass
 
-    def work(self, burst=False, logging_level="INFO", date_format=DEFAULT_LOGGING_DATE_FORMAT,
-             log_format=DEFAULT_LOGGING_FORMAT, max_jobs=None, with_scheduler=False):
+    def work(self, burst: bool = False, logging_level: str = "INFO", date_format=DEFAULT_LOGGING_DATE_FORMAT,
+             log_format=DEFAULT_LOGGING_FORMAT, max_jobs=None, with_scheduler: bool = False):
         """Starts the work loop.
 
         Pops and performs all jobs on the current list of queues.  When all
@@ -694,7 +720,7 @@ class Worker:
         self.heartbeat()
         return result
 
-    def heartbeat(self, timeout=None, pipeline=None):
+    def heartbeat(self, timeout=None, pipeline: t.Optional['Pipeline'] = None):
         """Specifies a new worker timeout, typically by extending the
         expiration time of the worker, effectively making this a "heartbeat"
         to not expire the worker until the timeout passes.
@@ -752,11 +778,11 @@ class Worker:
                                             job_class=self.job_class, serializer=self.serializer)
                            for queue in queues.split(',')]
 
-    def increment_failed_job_count(self, pipeline=None):
+    def increment_failed_job_count(self, pipeline: t.Optional['Pipeline'] = None):
         connection = pipeline if pipeline is not None else self.connection
         connection.hincrby(self.key, 'failed_job_count', 1)
 
-    def increment_successful_job_count(self, pipeline=None):
+    def increment_successful_job_count(self, pipeline: t.Optional['Pipeline'] = None):
         connection = pipeline if pipeline is not None else self.connection
         connection.hincrby(self.key, 'successful_job_count', 1)
 
@@ -764,7 +790,7 @@ class Worker:
         pipeline.hincrbyfloat(self.key, 'total_working_time',
                               job_execution_time.total_seconds())
 
-    def fork_work_horse(self, job, queue):
+    def fork_work_horse(self, job: 'Job', queue: 'Queue'):
         """Spawns a work horse to perform the actual work and passes it a job.
         """
         child_pid = os.fork()
@@ -778,14 +804,14 @@ class Worker:
             self._horse_pid = child_pid
             self.procline('Forked {0} at {1}'.format(child_pid, time.time()))
 
-    def get_heartbeat_ttl(self, job):
+    def get_heartbeat_ttl(self, job: 'Job'):
         if job.timeout and job.timeout > 0:
             remaining_execution_time = job.timeout - self.current_job_working_time
             return min(remaining_execution_time, self.job_monitoring_interval) + 60
         else:
             return self.job_monitoring_interval + 60
 
-    def monitor_work_horse(self, job, queue):
+    def monitor_work_horse(self, job: 'Job', queue: 'Queue'):
         """The worker will monitor the work horse and make sure that it
         either executes successfully or the status of the job is set to
         failed
@@ -804,7 +830,7 @@ class Worker:
                 self.set_current_job_working_time((utcnow() - job.started_at).total_seconds())
 
                 # Kill the job from this side if something is really wrong (interpreter lock/etc).
-                if job.timeout != -1 and self.current_job_working_time > (job.timeout + 60):
+                if job.timeout != -1 and self.current_job_working_time > (job.timeout + 60):  # type: ignore
                     self.heartbeat(self.job_monitoring_interval + 60)
                     self.kill_horse()
                     self.wait_for_horse()
@@ -833,7 +859,7 @@ class Worker:
 
         if job_status is None:  # Job completed and its ttl has expired
             return
-        elif job_status == JobStatus.STOPPED:
+        elif self._stopped_job_id == job.id:
             # Work-horse killed deliberately
             self.log.warning('Job stopped by user, moving job to FailedJobRegistry')
             self.handle_job_failure(
@@ -856,7 +882,7 @@ class Worker:
                            "(waitpid returned %s)" % ret_val
             )
 
-    def execute_job(self, job, queue):
+    def execute_job(self, job: 'Job', queue: 'Queue'):
         """Spawns a work horse to perform the actual work and passes it a job.
         The worker will wait for the work horse and make sure it executes
         within the given timeout bounds, or will end the work horse with
@@ -867,7 +893,7 @@ class Worker:
         self.monitor_work_horse(job, queue)
         self.set_state(WorkerStatus.IDLE)
 
-    def maintain_heartbeats(self, job):
+    def maintain_heartbeats(self, job: 'Job'):
         """Updates worker and job's last heartbeat field. If job was
         enqueued with `result_ttl=0`, a race condition could happen where this heartbeat
         arrives after job has been deleted, leaving a job key that contains only
@@ -888,7 +914,7 @@ class Worker:
             if results[2] == 1:
                 self.connection.delete(job.key)
 
-    def main_work_horse(self, job, queue):
+    def main_work_horse(self, job: 'Job', queue: 'Queue'):
         """This is the entry point of the newly spawned work horse."""
         # After fork()'ing, always assure we are generating random sequences
         # that are different from the worker.
@@ -916,7 +942,7 @@ class Worker:
         signal.signal(signal.SIGINT, signal.SIG_IGN)
         signal.signal(signal.SIGTERM, signal.SIG_DFL)
 
-    def prepare_job_execution(self, job):
+    def prepare_job_execution(self, job: 'Job'):
         """Performs misc bookkeeping like updating states prior to
         job execution.
         """
@@ -935,13 +961,15 @@ class Worker:
         msg = 'Processing {0} from {1} since {2}'
         self.procline(msg.format(job.func_name, job.origin, time.time()))
 
-    def handle_job_failure(self, job, queue, started_job_registry=None,
+    def handle_job_failure(self, job: 'Job', queue: 'Queue', started_job_registry=None,
                            exc_string=''):
-        """Handles the failure or an executing job by:
+        """
+        Handles the failure or an executing job by:
             1. Setting the job status to failed
             2. Removing the job from StartedJobRegistry
             3. Setting the workers current job to None
             4. Add the job to FailedJobRegistry
+        `save_exc_to_job` should only be used for testing purposes
         """
         self.log.debug('Handling failed execution of job %s', job.id)
         with self.connection.pipeline() as pipeline:
@@ -952,7 +980,6 @@ class Worker:
                     job_class=self.job_class,
                     serializer=self.serializer
                 )
-            job.worker_name = None
 
             # check whether a job was stopped intentionally and set the job
             # status appropriately if it was this job.
@@ -972,8 +999,14 @@ class Worker:
             if not self.disable_default_exception_handler and not retry:
                 failed_job_registry = FailedJobRegistry(job.origin, job.connection,
                                                         job_class=self.job_class, serializer=job.serializer)
+                # Exception should be saved in job hash if server
+                # doesn't support Redis streams
+                _save_exc_to_job = not self.supports_redis_streams
                 failed_job_registry.add(job, ttl=job.failure_ttl,
-                                        exc_string=exc_string, pipeline=pipeline)
+                                        exc_string=exc_string, pipeline=pipeline,
+                                        _save_exc_to_job=_save_exc_to_job)
+                if self.supports_redis_streams:
+                    Result.create_failure(job, job.failure_ttl, exc_string=exc_string, pipeline=pipeline)
                 with suppress(redis.exceptions.ConnectionError):
                     pipeline.execute()
 
@@ -999,7 +1032,7 @@ class Worker:
                 # even if Redis is down
                 pass
 
-    def handle_job_success(self, job, queue, started_job_registry):
+    def handle_job_success(self, job: 'Job', queue: 'Queue', started_job_registry: StartedJobRegistry):
         self.log.debug('Handling successful execution of job %s', job.id)
 
         with self.connection.pipeline() as pipeline:
@@ -1008,23 +1041,33 @@ class Worker:
                     # if dependencies are inserted after enqueue_dependents
                     # a WatchError is thrown by execute()
                     pipeline.watch(job.dependents_key)
-                    # enqueue_dependents calls multi() on the pipeline!
+                    # enqueue_dependents might call multi() on the pipeline
                     queue.enqueue_dependents(job, pipeline=pipeline)
 
+                    if not pipeline.explicit_transaction:
+                        # enqueue_dependents didn't call multi after all!
+                        # We have to do it ourselves to make sure everything runs in a transaction
+                        pipeline.multi()
+
                     self.set_current_job_id(None, pipeline=pipeline)
                     self.increment_successful_job_count(pipeline=pipeline)
                     self.increment_total_working_time(
-                        job.ended_at - job.started_at, pipeline
+                        job.ended_at - job.started_at, pipeline  # type: ignore
                     )
 
                     result_ttl = job.get_result_ttl(self.default_result_ttl)
                     if result_ttl != 0:
                         self.log.debug('Setting job %s status to finished', job.id)
                         job.set_status(JobStatus.FINISHED, pipeline=pipeline)
-                        job.worker_name = None
-                        # Don't clobber the user's meta dictionary!
-                        job.save(pipeline=pipeline, include_meta=False)
-
+                        # Result should be saved in job hash only if server
+                        # doesn't support Redis streams
+                        include_result = not self.supports_redis_streams
+                        # Don't clobber user's meta dictionary!
+                        job.save(pipeline=pipeline, include_meta=False,
+                                 include_result=include_result)
+                        if self.supports_redis_streams:
+                            Result.create(job, Result.Type.SUCCESSFUL, return_value=job._result,
+                                          ttl=result_ttl, pipeline=pipeline)
                         finished_job_registry = queue.finished_job_registry
                         finished_job_registry.add(job, result_ttl, pipeline)
 
@@ -1039,7 +1082,7 @@ class Worker:
                 except redis.exceptions.WatchError:
                     continue
 
-    def execute_success_callback(self, job, result):
+    def execute_success_callback(self, job: 'Job', result):
         """Executes success_callback with timeout"""
         job.heartbeat(utcnow(), CALLBACK_TIMEOUT)
         with self.death_penalty_class(CALLBACK_TIMEOUT, JobTimeoutException, job_id=job.id):
@@ -1051,7 +1094,7 @@ class Worker:
         with self.death_penalty_class(CALLBACK_TIMEOUT, JobTimeoutException, job_id=job.id):
             job.failure_callback(job, self.connection, *sys.exc_info())
 
-    def perform_job(self, job, queue):
+    def perform_job(self, job: 'Job', queue: 'Queue'):
         """Performs the actual work of a job.  Will/should only be called
         inside the work horse's process.
         """
@@ -1119,8 +1162,9 @@ class Worker:
 
         return True
 
-    def handle_exception(self, job, *exc_info):
+    def handle_exception(self, job: 'Job', *exc_info):
         """Walks the exception handler stack to delegate exception handling."""
+
         exc_string = ''.join(traceback.format_exception(*exc_info))
 
         # If the job cannot be deserialized, it will raise when func_name or
@@ -1132,14 +1176,17 @@ class Worker:
                 'arguments': job.args,
                 'kwargs': job.kwargs,
             }
+            func_name = job.func_name
         except DeserializationError:
             extra = {}
+            func_name = '<DeserializationError>'
 
         # the properties below should be safe however
         extra.update({'queue': job.origin, 'job_id': job.id})
-
+        
         # func_name
-        self.log.error(exc_string, exc_info=True, extra=extra)
+        self.log.error(f'[Job {job.id}]: exception raised while executing ({func_name})\n' + exc_string,
+                       extra=extra)
 
         for handler in self._exc_handlers:
             self.log.debug('Invoking exception handler %s', handler)
@@ -1199,13 +1246,13 @@ class Worker:
 
 
 class SimpleWorker(Worker):
-    def execute_job(self, job, queue):
+    def execute_job(self, job: 'Job', queue: 'Queue'):
         """Execute job in same thread/process, do not fork()"""
         self.set_state(WorkerStatus.BUSY)
         self.perform_job(job, queue)
         self.set_state(WorkerStatus.IDLE)
 
-    def get_heartbeat_ttl(self, job):
+    def get_heartbeat_ttl(self, job: 'Job'):
         # "-1" means that jobs never timeout. In this case, we should _not_ do -1 + 60 = 59.
         # # We should just stick to DEFAULT_WORKER_TTL.
         if job.timeout == -1:
